@@ -202,10 +202,16 @@ static XtResource appDataResourceList[] = {
    XtOffsetOf(AppData, compressLevel), XtRImmediate, (XtPointer) -1},
 
   {"qualityLevel", "QualityLevel", XtRInt, sizeof(int),
-   XtOffsetOf(AppData, qualityLevel), XtRImmediate, (XtPointer) -1},
+   XtOffsetOf(AppData, qualityLevel), XtRImmediate, (XtPointer) 6},
+
+  {"enableJPEG", "EnableJPEG", XtRBool, sizeof(Bool),
+   XtOffsetOf(AppData, enableJPEG), XtRImmediate, (XtPointer) True},
 
   {"useRemoteCursor", "UseRemoteCursor", XtRBool, sizeof(Bool),
    XtOffsetOf(AppData, useRemoteCursor), XtRImmediate, (XtPointer) True},
+
+  {"grabKeyboard", "GrabKeyboard", XtRBool, sizeof(Bool),
+   XtOffsetOf(AppData, grabKeyboard), XtRImmediate, (XtPointer) False},
 };
 
 
@@ -229,6 +235,7 @@ XrmOptionDescRec cmdLineOptions[] = {
   {"-depth",         "*requestedDepth",     XrmoptionSepArg, 0},
   {"-compresslevel", "*compressLevel",      XrmoptionSepArg, 0},
   {"-quality",       "*qualityLevel",       XrmoptionSepArg, 0},
+  {"-nojpeg",        "*enableJPEG",         XrmoptionNoArg,  "False"},
   {"-nocursorshape", "*useRemoteCursor",    XrmoptionNoArg,  "False"},
 };
 
@@ -277,9 +284,10 @@ void
 usage(void)
 {
   fprintf(stderr,
-	  "TightVNC viewer version 1.2.3 (based on VNC 3.3.3r2)\n"
+	  "TightVNC viewer version 1.2.7\n"
 	  "\n"
 	  "Usage: %s [<OPTIONS>] [<HOST>][:<DISPLAY#>]\n"
+	  "       %s [<OPTIONS>] [<HOST>][::<PORT#>]\n"
 	  "       %s [<OPTIONS>] -listen [<DISPLAY#>]\n"
 	  "       %s -help\n"
 	  "\n"
@@ -298,10 +306,12 @@ usage(void)
 	  "        -depth <DEPTH>\n"
 	  "        -compresslevel <COMPRESS-VALUE> (0..9: 0-fast, 9-best)\n"
 	  "        -quality <JPEG-QUALITY-VALUE> (0..9: 0-low, 9-high)\n"
+	  "        -nojpeg\n"
 	  "        -nocursorshape\n"
 	  "\n"
-	  "Option names may be abbreviated, e.g. -bgr instead of -bgr233."
-	  "\n", programName, programName, programName);
+	  "Option names may be abbreviated, e.g. -bgr instead of -bgr233.\n"
+	  "See the manual page for more information."
+	  "\n", programName, programName, programName, programName);
   exit(1);
 }
 
@@ -316,7 +326,8 @@ void
 GetArgsAndResources(int argc, char **argv)
 {
   int i;
-  char *vncServerName;
+  char *vncServerName, *colonPos;
+  int len, portOffset;
 
   /* Turn app resource specs into our appData structure for the rest of the
      program to use */
@@ -360,16 +371,25 @@ GetArgsAndResources(int argc, char **argv)
     exit(1);
   }
 
-  for (i = 0; vncServerName[i] != ':' && vncServerName[i] != 0; i++);
-
-  strncpy(vncServerHost, vncServerName, i);
-
-  if (vncServerName[i] == ':') {
-    vncServerPort = atoi(&vncServerName[i+1]);
+  colonPos = strchr(vncServerName, ':');
+  if (colonPos == NULL) {
+    /* No colon -- use default port number */
+    strcpy(vncServerHost, vncServerName);
+    vncServerPort = SERVER_PORT_OFFSET;
   } else {
-    vncServerPort = 0;
+    memcpy(vncServerHost, vncServerName, colonPos - vncServerName);
+    vncServerHost[colonPos - vncServerName] = '\0';
+    len = strlen(colonPos + 1);
+    portOffset = SERVER_PORT_OFFSET;
+    if (colonPos[1] == ':') {
+      /* Two colons -- interpret as a port number */
+      colonPos++;
+      len--;
+      portOffset = 0;
+    }
+    if (!len || strspn(colonPos + 1, "0123456789") != len) {
+      usage();
+    }
+    vncServerPort = atoi(colonPos + 1) + portOffset;
   }
-
-  if (vncServerPort < 100)
-    vncServerPort += SERVER_PORT_OFFSET;
 }
